﻿/******************************** Module Header ********************************
 * Module Name: DataProvider.cs
 * Project:     CommonLibrary
 * Copyright (c) Jackson Huang.
 * 
 * Code Logic:
 * 
 * 
 * Corresponding Source:
 * 
 * 
 * History:
 * 2011-9-4 11:09:20 Jackson Huang Created
 * Websiet:
 * http://www.cnblogs.com/rush/
 * *******************************************************************************/


using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data.Common;
using System.Linq;
using System.Text;
using System.Data;
using System.Data.SqlClient;
using System.Configuration.Provider;
using CommonLibrary.Data.SqlServer;

namespace CommonLibrary.Provider
{
    /// <summary>
    /// Defines the methods that Data providers should be implemented.
    /// </summary>
    public abstract class DataProvider : ProviderBase
    {


        /// <summary>
        /// Runs the specified transaction.
        /// </summary>
        /// <param name="transaction">The transaction.</param>
        public abstract void Run(DbTransaction transaction);


        /// <summary>
        /// Runs the specified transaction.
        /// </summary>
        /// <param name="transaction">The transaction.</param>
        /// <param name="parameters">The stored procedure  parameters.</param>
        public abstract void Run(DbTransaction transaction, DbParameter[] parameters);


        //public abstract DbDataReader Run(DbParameter[] parameters);


        /// <summary>
        /// Runs the specified connection string.
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        /// <param name="parameters">The stored procedure  parameters.</param>
        /// <returns>Returns dataset.</returns>
        public abstract DataSet Run(string connectionString, DbParameter[] parameters);


        /// <summary>
        /// Runs the scalar.
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        /// <param name="parameters">The stored procedure  parameters.</param>
        /// <returns>Returns an object.</returns>
        public abstract object RunScalar(string connectionString, DbParameter[] parameters);


        /// <summary>
        /// Runs the scalar.
        /// </summary>
        /// <param name="transaction">The transaction.</param>
        /// <param name="parameters">The stored procedure  parameters.</param>
        /// <returns></returns>
        public abstract object RunScalar(SqlTransaction transaction, DbParameter[] parameters);


        /// <summary>
        /// Runs the specified connectionstring.
        /// </summary>
        /// <param name="connectionstring">The connectionstring.</param>
        /// <returns></returns>
        public abstract DataSet Run(string connectionstring);


        /// <summary>
        /// Runs this instance.
        /// </summary>
        public abstract void Run();


        /// <summary>
        /// Runs the specified parameters.
        /// </summary>
        /// <param name="parameters">The stored procedure  parameters.</param>
        /// <returns></returns>
        public abstract DataSet Run(DbParameter[] parameters);


        /// <summary>
        /// Runs the specified command type.
        /// </summary>
        /// <param name="commandType">Type of the command.</param>
        /// <param name="commandText">The command text stored procedure or sql queries.</param>
        /// <returns></returns>
        public abstract DbDataReader Run(CommandType commandType, string commandText);


        /// <summary>
        /// Gets or sets the stored procedure parameters.
        /// </summary>
        /// <value>
        /// The stored procedure parameters.
        /// </value>
        public DbParameter[] Parameters { get; set; }


        /// <summary>
        /// Gets or sets the name of the stored procedure.
        /// </summary>
        /// <value>
        /// The name of the stored procedure.
        /// </value>
        public string StoredProcedureName { get; set; }


        /// <summary>
        /// Gets the default connection string from webconfig file.
        /// </summary>
        protected string ConnectionString
        {
            get { return ConfigurationManager.ConnectionStrings["SQLCONN"].ToString(); }
        }
    }
}
