﻿/******************************** Module Header ********************************
 * Module Name: DataProviderManager.cs
 * Project:     EnvDTE.SolutionClass
 * Copyright (c) Jackson Huang.
 * 
 * Code Logic:
 * 
 * 
 * Corresponding Source:
 * 
 * 
 * History:
 * 5-09-2011 20:05 Jackson Huang Created
 * Websiet:
 * http://www.cnblogs.com/rush/
 * *******************************************************************************/

using System;
using System.Collections.Generic;
using System.Configuration.Provider;
using System.Linq;
using System.Text;
using System.Data.Common;
using System.Web.Configuration;

namespace CommonLibrary.Provider
{
    /// <summary>
    /// The class maintains provider.
    /// </summary>
    public static class DataProviderManager
    {
        private static DataProvider _provider = null;
        private static DataProviderCollection _providers = null;

        //Providers should be free-threaded/thread safe, with one instance per application domain.
        private static readonly object SyncLock = new object();


        /// <summary>
        /// The factory method to creates the provider instance.
        /// </summary>
        /// <returns>The instances of provider.</returns>
        public static object CreateProvider()
        {
            LoadProviders();

            return _provider;
        }


        /// <summary>
        /// The factory method to creates the provider instance.
        /// </summary>
        /// <param name="storedProcedureName">Name of the stored procedure.</param>
        /// <param name="parameters">Name of the stored procedure parameters.</param>
        /// <returns>The instance of provider.</returns>
        public static object CreateProvider(string storedProcedureName, DbParameter[] parameters)
        {
            LoadProviders();

            _provider.StoredProcedureName = storedProcedureName;
            _provider.Parameters = parameters;

            return _provider;
        }


        /// <summary>
        /// Loads the providers by webconfig setting.
        /// </summary>
        private static void LoadProviders()
        {
            // providers are loaded just once
            if (null == _providers)
            {
                // Synchronize the process of loading the providers
                lock (SyncLock)
                {
                    // Double confirm that the _provider is still null.
                    if (null == _provider)
                    {
                        try
                        {
                            // Reads the webconfig file corresponding node.
                            DataProviderSection section = (DataProviderSection)
                                                          WebConfigurationManager.GetSection("system.web/dataProviderService");

                            _providers = new DataProviderCollection();

                            // Creates provider instance, and invokes ProviderBase's Initialize function.
                            ProvidersHelper.InstantiateProviders(section.Providers, Providers, typeof(DataProvider));

                            // Gets the default in the collection.
                            _provider = Providers[section.DefaultProvider];
                        }
                        catch
                        {
                            throw new ProviderException("Can't create instance");
                        }
                    }
                }
            }
        }


        /// <summary>
        /// Gets the single provider.
        /// </summary>
        public static DataProvider Provider
        {
            get { return _provider; }
        }


        /// <summary>
        /// Gets the collection of providers.
        /// </summary>
        public static DataProviderCollection Providers
        {
            get { return _providers; }
        }

    }
}
