﻿/******************************** Module Header ********************************
 * Module Name: SqlDataProvider.cs
 * Project:     EnvDTE.SolutionClass
 * Copyright (c) Jackson Huang.
 * 
 * Code Logic:
 * 
 * 
 * Corresponding Source:
 * 
 * 
 * History:
 * 5-09-2011 21:05 Jackson Huang Created
 * Websiet:
 * http://www.cnblogs.com/rush/
 * *******************************************************************************/

using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Data.Common;
using System.Data.SqlClient;
using System.Linq;
using System.Text;
using CommonLibrary.Data.SqlServer;

namespace CommonLibrary.Provider
{
    /// <summary>
    /// The implementor of DataProvder.
    /// </summary>
    public class SqlDataProvider : DataProvider
    {

        #region Fields

        private string _connection = string.Empty;

        #endregion

        /// <summary>
        /// Default parameterless constructor used by Reflection.
        /// </summary>
        public SqlDataProvider()
        {
            // Just used by Reflection.
        }

        /// <summary>
        /// Initializes the provider.
        /// </summary>
        /// <param name="name">The name of provider, setting in webconfig file.</param>
        /// <param name="config">The value collection of config.</param>
        /// <exception cref="T:System.ArgumentNullException">提供程序的名称是 null。</exception>
        ///   
        /// <exception cref="T:System.ArgumentException">提供程序的名称长度为零。</exception>
        ///   
        /// <exception cref="T:System.InvalidOperationException">提供程序初始化完成后，将尝试调用该提供程序上的 <see cref="M:System.Configuration.Provider.ProviderBase.Initialize(System.String,System.Collections.Specialized.NameValueCollection)"/>。</exception>
        public override void Initialize(string name, System.Collections.Specialized.NameValueCollection config)
        {
            // Due to ProviderBase has check name and config are available or not,
            // So no need validate name and config.
            base.Initialize(name, config);
            _connection = config["connectionString"];

            if (string.IsNullOrEmpty(_connection))
                throw new ConfigurationErrorsException("Connection string can't empty.");
        }


        /// <summary>
        /// Runs the specified transaction.
        /// </summary>
        /// <param name="transaction">The transaction.</param>
        public override void Run(DbTransaction transaction)
        {
            SqlHelper.ExecuteNonQuery(transaction as SqlTransaction,
                CommandType.StoredProcedure, StoredProcedureName, Parameters);
        }


        /// <summary>
        /// Runs the specified transaction.
        /// </summary>
        /// <param name="transaction">The transaction.</param>
        /// <param name="parameters">The parameters.</param>
        public override void Run(DbTransaction transaction, DbParameter[] parameters)
        {
            SqlHelper.ExecuteNonQuery(transaction as SqlTransaction,
                CommandType.StoredProcedure, StoredProcedureName, parameters as SqlParameter[]);
        }


        /// <summary>
        /// Runs the specified connection string.
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        public override DataSet Run(string connectionString, DbParameter[] parameters)
        {
            DataSet ds = SqlHelper.ExecuteDataset(connectionString, StoredProcedureName, parameters);
            return ds;
        }


        /// <summary>
        /// Runs the specified parameters.
        /// </summary>
        /// <param name="parameters">The stored procedure  parameters.</param>
        /// <returns></returns>
        public override DataSet Run(DbParameter[] parameters)
        {
            DataSet ds = SqlHelper.ExecuteDataset(ConnectionString, StoredProcedureName, parameters);

            return ds;

        }


        /// <summary>
        /// Runs the scalar.
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        public override object RunScalar(string connectionString, DbParameter[] parameters)
        {
            object obj = SqlHelper.ExecuteScalar(connectionString, StoredProcedureName, parameters);

            return obj;
        }


        /// <summary>
        /// Runs the scalar.
        /// </summary>
        /// <param name="transaction">The transaction.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        public override object RunScalar(SqlTransaction transaction, DbParameter[] parameters)
        {
            object obj = SqlHelper.ExecuteScalar(transaction, StoredProcedureName, parameters);

            return obj;
        }


        /// <summary>
        /// Runs the specified connectionstring.
        /// </summary>
        /// <param name="connectionstring">The connectionstring.</param>
        /// <returns></returns>
        public override DataSet Run(string connectionstring)
        {
            DataSet ds = SqlHelper.ExecuteDataset(connectionstring, 
                CommandType.StoredProcedure, StoredProcedureName);

            return ds;

        }


        /// <summary>
        /// Runs this instance.
        /// </summary>
        public override void Run()
        {
            SqlHelper.ExecuteNonQuery(ConnectionString,
                CommandType.StoredProcedure, StoredProcedureName, Parameters);
        }



        //public override DbDataReader Run(DbParameter[] parameters)
        //{
        //    SqlDataReader dr = SqlHelper.ExecuteReader(ConnectionString, CommandType.StoredProcedure,
        //                                               StoredProcedureName, parameters);

        //    return dr;

        //}

        /// <summary>
        /// Runs the specified parameters.
        /// </summary>
        /// <param name="commandType">Type of the command.</param>
        /// <param name="commandText">The command text stored procedure or sql queries.</param>
        /// <returns></returns>
        public override DbDataReader Run(CommandType commandType, string commandText)
        {
            SqlDataReader dr = SqlHelper.ExecuteReader(ConnectionString, CommandType.Text, commandText);

            return dr;

        }


        public SqlParameter[] Parameters { get; set; }
    }
}
